"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
class ActivationState {
    constructor() {
        this.beforeActivationIdx = 0;
        this.afterActivationIdx = 0;
    }
}
class Activation {
    async execute(ctx, onError) {
        // Allow the arguments to be set outside of any interceptor
        //    For example set them before we call execute. This is so we can use
        //    the activation outside of an environment using activation chains
        if (this.method.parameters.length > 0 && lodash_1.isNil(ctx.getArguments())) {
            ctx.setArguments(new Array(this.method.parameters.length));
        }
        const activationState = new ActivationState();
        ctx.setData('activation_activation_state', activationState);
        let beforeInterceptor = null;
        try {
            let resultValue = false;
            for (activationState.beforeActivationIdx = 0; activationState.beforeActivationIdx < this.beforeActivation.length; ++activationState.beforeActivationIdx) {
                beforeInterceptor = this.beforeActivation[activationState.beforeActivationIdx];
                const result = beforeInterceptor.before(ctx);
                resultValue = false;
                if (result instanceof Promise) {
                    resultValue = await result;
                }
                else {
                    resultValue = result;
                }
                // We do not continue with the processing
                if (resultValue === false) {
                    break;
                }
            }
        }
        catch (err) {
            if (lodash_1.isFunction(onError)) {
                onError(beforeInterceptor, err);
            }
            ctx.getResult().setError(err);
        }
        let afterActivation = null;
        for (activationState.afterActivationIdx = this.afterActivation.length - 1; activationState.afterActivationIdx >= 0; activationState.afterActivationIdx--) {
            afterActivation = this.afterActivation[activationState.afterActivationIdx];
            // We don't care about after activation result
            try {
                await afterActivation.after(ctx);
            }
            catch (err) {
                if (lodash_1.isFunction(onError)) {
                    onError(beforeInterceptor, err);
                }
                ctx.getResult().setError(err);
            }
        }
        return true;
    }
    removeBeforeActivation(activation, context) {
        const idx = this.beforeActivation.indexOf(activation);
        if (idx >= 0) {
            const actionData = context.getData('activation_activation_state');
            this.beforeActivation.splice(idx, 1);
            actionData.beforeActivationIdx--;
        }
    }
    removeAfterActivation(activation, context) {
        const idx = this.afterActivation.indexOf(activation);
        if (idx >= 0) {
            this.afterActivation.splice(idx, 1);
        }
    }
}
exports.Activation = Activation;
//# sourceMappingURL=activation.execution.js.map